// set the dimensions and margins of the graph
const width = 800;
const height = 500;

// append the svg object to the body of the page
const svg = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

const svg2 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

function render(mapData, countryData) {
  var projection = d3
    .geoEqualEarth()
    .fitSize([width / 2, height], mapData);
  var path = d3.geoPath(projection);

  const mapColorAttr = 'Birth Rate';
  var mapColorScale = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          countryData.values(),
          (r) => +r[mapColorAttr],
        ),
      ),
    )
    .range(['white', 'blue']);

  function getCountryColor(name) {
    let entry = countryData.get(name);
    if (!(entry == undefined)) {
      return mapColorScale(entry[mapColorAttr]);
    }
    return 'grey';
  }

  svg
    .append('g')
    .selectAll('path')
    .data(mapData.features)
    .join('path')
    // draw each country
    .attr('fill', function (d) {
      return getCountryColor(d.properties.name);
    })
    .on('mouseover', function (event, data) {
      //d3.select(this).attr('fill', 'red');
      d3.selectAll(`#${this.id}`)
        .attr('fill', 'red')
        .attr('r', 20)
        .selectAll('circle');
    })
    .on('mouseleave', function (event, data) {
      d3.selectAll(`#${this.id}`)
        .attr('fill', getCountryColor(data.properties.name))
        .attr('r', (d) => rScale(d[rAttr]));
    })
    .attr('id', (d) => d.properties.name)
    .attr('d', path);

  const xAttr = 'Fertility Rate';
  const x = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(countryData.values(), (d) => d[xAttr]),
      ),
    )
    .range([0, width / 2]);

  const yAttr = 'Life expectancy';
  const y = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(
          countryData.values().filter((d) => d[yAttr] > 0),
          (d) => d[yAttr],
        ),
      ),
    )
    .range([height, 0]);

  const plotColorAttr = 'Continent';
  const plotColorScale = d3.scaleOrdinal(
    d3.schemeCategory10,
  );
  function getPointColor(d) {
    return plotColorScale(d[plotColorAttr]);
  }

  const rAttr = 'Population';
  const rScale = d3
    .scaleLinear()
    .domain(
      d3.extent(
        d3.map(countryData.values(), (d) => d[rAttr]),
      ),
    )
    .range([4, 15]);

  svg2
    .append('g')
    .attr('transform', `translate(0,${height - 50})`) // This controls the vertical position of the Axis
    .call(d3.axisBottom(x));

  svg2
    .append('g')
    .attr('transform', `translate(0,0)`) // This controls the vertical position of the Axis
    .call(d3.axisRight(y));

  svg2
    .append('g')
    .selectAll('circle')
    .data(countryData.values().filter((c) => +c[yAttr] > 0))
    .join('circle')
    .attr('cx', (d) => x(d[xAttr]))
    .attr('cy', (d) => y(d[yAttr]))
    .attr('r', (d) => rScale(d[rAttr]))
    .attr('fill', getPointColor)
    .attr('id', (d) => d.Country)
    .on('mouseover', function (event, data) {
      //d3.select(this).attr('fill', 'red');
      d3.selectAll(`#${this.id}`)
        .attr('fill', 'red')
        .attr('r', 20)
        .selectAll('circle');
    })
    .on('mouseleave', function (event, data) {
      d3.selectAll(`#${this.id}`)
        .attr('fill', getCountryColor(data.Country))
        .attr('r', (d) => rScale(d[rAttr]));
    });
}

//Read the data
const mapUrl =
  'https://raw.githubusercontent.com/holtzy/D3-graph-gallery/master/DATA/world.geojson';
const countryStatsUrl =
  'https://raw.githubusercontent.com/bepnye/CP341/main/world-data-2023.csv';
const continentUrl =
  'https://raw.githubusercontent.com/samayo/country-json/master/src/country-by-continent.json';

function normalizeCountryName(s) {
  return s.replaceAll(' ', '_').toLowerCase();
}

Promise.all([
  d3.json(mapUrl),
  d3.csv(countryStatsUrl),
  d3.json(continentUrl),
]).then(([mapData, csvData, continentData]) => {
  var countryData = new Map();
  csvData.forEach((row) => {
    row.Country = normalizeCountryName(row.Country);
    row.Population = +row.Population.replaceAll(',', '');
    row.Continent = 'Unknown';
    countryData.set(row.Country, row);
  });
  mapData.features.forEach((f) => {
    f.properties.name = normalizeCountryName(
      f.properties.name,
    );
  });
  continentData.forEach((row) => {
    let name = normalizeCountryName(row.country);
    if (countryData.has(name)) {
      countryData.get(name).Continent = row.continent;
    } else {
      console.log('No match found for ', name);
    }
  });
  console.log(countryData);
  render(mapData, countryData);
});
